module Main where

import Data.List
import Control.Monad.State
import System.Random
import System.Directory
import Data.Char

-- Tarefa 1: Monte Carlo - estimar pi

estimaPi :: Int -> Double
estimaPi n = 4 * tot / fromIntegral n
  where
    xs = [0.0, 0.01 .. 1.0]
    ys = [0.0, 0.01 .. 1.0]
    coords = pure (,) <*> xs <*> ys 
    tot = foldl' contaDentro 0 $ take n coords 

contaDentro acc xy
  | estaDentro xy = acc+1
  | otherwise     = acc

estaDentro (x,y) = (x-0.5)^2 + (y-0.5)^2 <= 0.25

type Aleatorio a = State StdGen a

rndDouble :: Aleatorio Double
rndDouble = state random  

rndCoord :: Aleatorio (Double, Double)
rndCoord = do x <- rndDouble 
              y <- rndDouble
              return (x,y) -- pure (x,y)

contaDentroRnd :: Int -> Aleatorio Double
contaDentroRnd n = go n 0
  where 
    go 0 acc = return acc -- pure acc 
    go n acc = do xy <- rndCoord 
                  go (n-1) (contaDentro acc xy)

estimaPiRnd :: Int -> Aleatorio Double
estimaPiRnd n = do cont <- contaDentroRnd n
                   return $ calc cont -- pure $ calc cont
  where 
    calc :: Double -> Double
    calc acc = 4*acc / fromIntegral n

-- Tarefa 2 : brincando com IO
listaArquivos :: FilePath -> IO [String]
listaArquivos fp = 
  do nomes <- listDirectory fp 
     let texto = filter (".txt" `isSuffixOf`) nomes 
     return texto 

imprimeArquivo :: FilePath -> IO String
imprimeArquivo fp = do conteudo <- readFile fp
                       putStrLn conteudo 
                       return conteudo

pipeline :: FilePath -> IO ()
pipeline fp = 
  do fps <- listaArquivos fp 
     textos <- mapM (imprimeArquivo . concatPath) fps 
     let textos' = map processaConteudo textos 
     mapM_ putStrLn textos'
  where 
    concatPath fname = fp ++ "/" ++ fname

processaConteudo :: String -> String
processaConteudo s = concatMap (show . sum . map ord) $ words s

main :: IO ()
main = do
  print $ estimaPi 1000
  g <- getStdGen
  print $ runState (estimaPiRnd 1000) g
  pipeline "textos"
